module IIR_Filter (
		 	clk,
		 	rstn,
		 	din_re,
		 	din_im,
			a_re,
			a_im,
			b_re,
			b_im,
			dout
);

input	       rstn;
input	       clk;
input   signed	[15:0] din_re;
input   signed	[15:0] din_im;
input   signed	[31:0] a_re;
input   signed	[31:0] a_im;
input   signed	[31:0] b_re;
input   signed	[31:0] b_im;

output  signed	[15:0] dout;

wire	signed	[48:0]	mult_x_re;
wire	signed	[48:0]	mult_x_im;
wire	signed	[54:0]	mult_y_re;
wire	signed	[54:0]	mult_y_im;
wire	signed	[15:0]	dout_t;
wire	signed	[50:0]	Ysum_re;
wire	signed	[50:0]	Ysum_im;

reg	signed	[15:0]  dout_r1;
reg	signed	[50:0]	YsumR_re;
reg	signed	[50:0]	YsumR_im;
reg	signed	[50:0]	YsumR1_re;
reg	signed	[50:0]	YsumR1_im;



mult_C	#(16,16,32,32)	inst_c1			(	.a		(din_re		),
							.b		(din_im		),
							.c		(a_re		),
							.d		(a_im		),
							.Re		(mult_x_re	),
							.Im		(mult_x_im	)
				);


mult_C	#(32,32,32,32)	inst_c2			(	.a		(YsumR_re	),
							.b		(YsumR_im	),
							.c		(b_re		),
							.d		(b_im		),
							.Re		(mult_y_re	),
							.Im		(mult_y_im	)
				);


assign	Ysum_re       =   mult_x_re - mult_y_re;
assign	Ysum_im       =   mult_x_im - mult_y_im;


always @(posedge clk or negedge rstn) 
  	if (!rstn)
		 begin
		    YsumR_re     <= 'h0;
		    YsumR_im     <= 'h0;
		 end 
	 else 
		 begin
		    YsumR_re   <= {{20{Ysum_re[50]}},Ysum_re[50:20]} + Ysum_re[50];
		    YsumR_im   <= {{20{Ysum_im[50]}},Ysum_im[50:20]} + Ysum_im[50];
		  end

always @(posedge clk or negedge rstn) 
  	if (!rstn)
		 begin
		    YsumR1_re    <= 'h0;
		 end 
	 else 
		 begin
		    YsumR1_re  <= {{16{YsumR_re[50]}},YsumR_re[50:16]};
		  end

always @(posedge clk or negedge rstn) 
  	if (!rstn)
		 begin
		    dout_r1 <= 'h0;
		 end 
	 else 
		 begin

	    	if(YsumR1_re[16:15]==2'b01)
			dout_r1	<=	16'd32767;
		    else if(YsumR1_re[16:15]==2'b10)
			dout_r1	<=	-16'd32768;
		    else
			dout_r1	<=	YsumR1_re[15:0];
		 
		  end

assign	dout = dout_r1;

endmodule
